#-----------------------------------------------------------
#            simulator.tcl -- build a simulator
#	 	   Author: Ping Yang
#                       08/2000
#-----------------------------------------------------------


#load the XSB interface for TCL
source "$xmc_path/GUI/xsb_interface.tcl"

if {$tcl_platform(platform) == "unix"} {
    source "$xmc_path/GUI/mclistbox.tcl"
}

package require mclistbox 1.02
catch {namespace import mclistbox::*}

global resel_states state_num

set resel_states 0
set state_num 0

# ----------------------------------------------------------------------
proc simulation_start {} {
    global xsb_exec xmc_path
    global source_name 

    # disable "Start..." and enable "Stop..."
    .mb menuconfigure .simulation.starts -state disabled
    .mb menuconfigure .simulation.stops -state normal

    # create simulation window
    toplevel .simulation
    wm title .simulation "Interactive Simulation"
    wm geometry .simulation +250+250

    set sim .simulation
    # First, a message box shows the status of simulation
    iwidgets::messagebox $sim.mb -labeltext "Interactive Simulation Status" \
	-hscrollmode dynamic \
	-vscrollmode dynamic \
	-labelpos n -visibleitems 60x10
    pack $sim.mb -padx 4 -pady 4 -fill both -expand yes

    $sim.mb type add ERROR -background white -foreground red \
	-font "Helvetica -14"
    $sim.mb type add INFO -background white -foreground black \
	-font "Helvetica -14"
    $sim.mb type add ANSWER -background white -foreground blue \
	-font "Helvetica -14 bold"

    set sbt [frame .simulation.buttons]
    pack $sbt -padx 4 -pady 4 -side bottom
    button $sbt.cancel -text "Cancel" -command {simulation_stop}
    pack $sbt.cancel -padx 4 -pady 4 -side right -expand 1
    
    # initialize XSB and XMC
    $sim.mb issue "Initializing XMC ..." INFO
    update
    xsb_init "$xsb_exec" "\['$xmc_path/System/tcl_interface'\]"
    xsb_command "\['$xmc_path/System/xmc'\]"
    xsb_command {[gui]}
    xsb_command {[simgui]}

    # Compile the .xl file & start on XMC checking
    $sim.mb issue "Compiling the source file ..." INFO
    update
    set xlfile [string trimright $source_name ".xl"]
    xsb_query xmc_compile(\'$xlfile\',ErrorFile) result
    set errfile $result(1,1)
    if {$result(1,1) != "true"} {
	$sim.mb issue "Error in source file: " ERROR
	# display error file
	set f [open $result(1,1) "r"]
	for {set line 1} {! [eof $f]} {incr line} {
           $sim.mb issue [gets $f] INFO
	}
	close $f
	return
    }

    # start simulation
    $sim.mb issue "Start Interactive Simulation" INFO
    destroy $sim
    simulation
}


proc simulation_stop {} {
    global _node src_area
    global resel_states state_num

    xsb_close

    $src_area tag delete LINEdim CHARdim LINEbright CHARbright

    catch { destroy .hlp}
    catch { destroy .hlp1}
    catch { destroy .selp}
    catch { destroy .input_param}
    catch { destroy .prompt}
    catch { destroy .simulation }
    catch { destroy .simulatortree}
    catch { destroy .choice}
    catch { unset _node}

    set resel_states 0
    set state_num 0

    # disable "Start..." and enable "Stop..."
    .mb menuconfigure .simulation.starts -state normal
    .mb menuconfigure .simulation.stops -state disabled
}


# Simulation start, prompt user to select a process to be simulated
proc simulation {} {
    global procname	
    option add *textBackground seashell

    catch {destroy .selp}
    set w [toplevel .selp]
    wm title $w "Select process"
    wm geometry $w +250+250

    iwidgets::optionmenu $w.cb -labeltext "Processes:" -foreground blue 
    pack $w.cb -side top -pady 3m

    xsb_query get_processes(Procname) processes
    for {set i 1} {$i <= $processes(ntuples) } {incr i} {
        $w.cb insert end $processes($i,1)
    }

    button $w.ok -text "OK" -command {set procname [.selp.cb get]; destroy .selp; input_params}
    pack $w.ok -side left -expand 1
    button $w.cancel -text "Cancel" -command simulation_stop 
    pack $w.cancel -side left -expand 1        
}


# Prompt user to input the variables in the parameters
proc input_params {} {
    global procname para_list
    global pnamelist chan_num

    xsb_query get_pchans($procname,Channame) chans
    xsb_query get_pvars($procname,Varname) vars

    set chan_num $chans(ntuples)
    set var_num $vars(ntuples)
    set para_list {}
    set i 0

    # Three cases of parameters: 1. no channel, no variable
    # 2. has channel  3. has variable
    if {$chan_num == 0 && $var_num == 0} {
	set pnamelist $procname
	new_hierarchy
    } else {
	catch {destroy .input_param}
	toplevel .input_param
	wm title .input_param $procname
	wm geometry .input_param +250+250

	set enter .input_param.enter
	label $enter -text "Parameters:" -font "Helvetica -16" \
	    -foreground black 
	pack $enter -side top

	frame .input_param.raise -relief groove -borderwidth 1.5m 
	pack .input_param.raise -side top -pady 2m

	if {$chan_num != 0} {
	    set cf [frame .input_param.raise.framec]
	    pack $cf -padx 4 -pady 4 -side top

	    set clabels $cf.lables
	    label $clabels -text "Channels:                             " \
		-font "Helvetica -14" -foreground blue 
	    pack $clabels -side left 

	    for {set j 1} {$j <= $chan_num} {incr j} {
	   	set cw [frame $cf$i]
		pack $cw -padx 4 -pady 4 -side top

		set cvlabels $cw.lables$i
		label $cvlabels -text $chans($j,1) -width 8\
		    -foreground black -anchor w
		pack $cvlabels -side left

		set i [expr $i + 1]
	    }
	}

	if {$var_num != 0} { 
	    set vf [frame .input_param.raise.framev]
	    pack $vf -padx 4 -pady 4 -side top

	    set vlabels $vf.labels
	    label $vlabels -text "Variables:                            " \
		-font "Helvetica -14" -foreground blue -anchor nw
	    pack $vlabels -side left

	    set inpframev .input_param.raise.framev
	    for {set j 1} {$j <= $var_num} {incr j} {
		set vw [frame $vf$i]
		pack $vw -padx 4 -pady 4 -side top

		set vvlabels $vw.lables$i
		label $vvlabels -text $vars($j,1) -width 6\
		    -foreground black -anchor w
		pack $vvlabels -side left
		set i [expr $i + 1]

		set entryname $vw.entry$i
                lappend para_list $entryname
                iwidgets::entryfield $entryname -insertbackground red\
                	-labelpos w  -foreground black -width 10
                pack $entryname -side left 

#                set labels $vw.lables$i
#                label $labels -text $vars($j,2) 
#                pack $labels -side right	
                set i [expr $i + 1]
	    }
        }

        set btn [frame .input_param.buttons]
        pack $btn -side bottom -fill x -pady 4m
        button $btn.ok -text "OK" -command inputp_ok 
        pack $btn.ok -side left -expand 1
	button $btn.cancel -text "Cancel" -command simulation_stop
	pack $btn.cancel -side left -expand 1
	button $btn.hlp -text "Help" -command hlp_params -foreground red 
	pack $btn.hlp -side left -expand 1
    }
}

# help file in how to input parameters
proc hlp_params {} {
    global xmc_path

    catch {destroy .hlp}
    set hlp [iwidgets::hyperhelp .hlp -title "help for parameters" \
		 -modality none -topics {hlp_inp} \
		 -helpdir [file join $xmc_path help]]
    wm geometry .hlp +700+400
    $hlp showtopic hlp_inp
    $hlp activate
}


# After user input the parameters. If there is no input and user press 'ok'
# there is no response
proc inputp_ok {} {
    global procname chan_num
    global para_list pnamelist

    set dflag 1
    set pnamelist $procname
      
    for {set i 0} {$i < [llength $para_list] } {incr i} {
        set para [[lindex $para_list $i] get]
        if {$para == ""} {
             focus [[lindex $para_list $i] component entry]
                 set dflag 0
                 break
        }
   }

    if {$dflag == 1} {
	append pnamelist (
	if {$chan_num != 0} {
	    for {set i 1} {$i <= [expr $chan_num - 1]} {incr i} {
		append pnamelist $i ,
	    }
	    append pnamelist $i
	    if {[llength $para_list]!=0} {
		append pnamelist ,
	    }
	}
	for {set i 0} {$i < [llength $para_list] } {incr i} {
	    append pnamelist [[lindex $para_list $i] get] 
	    if {$i != [llength $para_list] -1 } {	
		append pnamelist ,
	    }
	}
	append pnamelist )
        new_hierarchy 
    }
}


# Show the simulator tree
proc new_hierarchy {} {
    global simTree simProcList
    global xmc_path 
    global sel_list

    set sel_list {}
    destroy .input_param
 
    set w [toplevel .simulatortree]
    wm title $w "Simulation Tree"
    wm geometry .simulatortree +250+250

    # load justifier's Prolog program
    xsb_command {[navigate,justify,annot,models_rule]}

    set pw $w.p
    iwidgets::panedwindow $pw -orient horizontal \
 	-width 480 -height 500
    pack $pw -side top -expand 1 -fill both

    set bitmap_path "$xmc_path/GUI"

    image create bitmap nodebmp  -file "$bitmap_path/node.bmp"
    image create bitmap openbmp  -file "$bitmap_path/open.bmp"
    image create bitmap closebmp -file "$bitmap_path/close.bmp"

    image creat photo iconreselection -file "$bitmap_path/branch.gif"

    $pw add "simulation"
    set simTree [$pw childsite "simulation"].stree

    iwidgets::hierarchy $simTree -labeltext "Simulation Tree" \
        -querycommand "query_sim_node %n" \
	-alwaysquery 1 \
        -visibleitems 60x20 \
        -borderwidth 1 -tabs 12 \
        -hscrollmode dynamic \
        -vscrollmode dynamic \
        -sbwidth 10 -scrollmargin 0 \
        -nodeicon nodebmp \
        -openicon openbmp \
        -closedicon closebmp \
	-iconcommand "reselect_node %n" \
        -selectcommand "select_sim_node %n" \
	-selectbackground lightgray \
        -foreground "black" \
        -font "Helvetica -14"
    pack $simTree -side top -expand yes -fill both

    $simTree component itemMenu add command -label "Reselection" \
	-foreground blue \
        -command {reselect_node [$simTree current]}
    $simTree component bgMenu add command -label "Expand All" \
	-command expand_all
    $simTree component bgMenu add command -label "Collapse All" \
	-command collapse_all

    # The process list
    $pw add "proc"
    set simProcList [$pw childsite "proc"].proclist
    mclistbox::mclistbox $simProcList \
        -bd 0 \
        -height 6 -width 60 \
        -columnrelief flat \
        -labelanchor w \
        -columnborderwidth 0 \
        -labelborderwidth 2 \
        -fillcolumn vars \
        -selectcommand highlight_sim_bright \
        -selectmode single

    $simProcList column add name -label "Process" -width 20
    $simProcList column add line -label ""      -width 0
    $simProcList column add char -label ""      -width 0
    $simProcList column add vars -label "Variables" -width 40
    pack $simProcList -side top -fill x -expand yes

    $pw fraction 70 30
    set btn [frame $w.buttons]
    pack $btn -padx 4 -pady 4 -side bottom
    button $btn.cancel -text "Cancel" -command simulation_stop \
	-bd 2 -relief groove
    pack $btn.cancel -padx 4 -pady 4 -side left -expand 1 -fill x

    button $btn.hlp -text "Help" -command hlp_simulation \
	-foreground red -bd 2 -relief groove
    pack $btn.hlp -padx 4 -pady 4 -side left -expand 1 -fill x
}


# help file in how to run the simulator tree
proc hlp_simulation {} {
    global xmc_path

    catch {destroy .hlp1}
    set hlp [iwidgets::hyperhelp .hlp1 \
		 -title "Help for Interactive Simulation" \
		 -modality none -topics {hlp_sim} \
		 -helpdir [file join $xmc_path help]]
    wm geometry .hlp1 +700+400
    $hlp showtopic hlp_sim
    $hlp activate
}


#-------------------------------------------------------------------
# Main part of simulation. We did selection and reselection
# It depends on the hierarchy structure of Tcl/Tk very much which 
# complicates the program structure.
#
# If there are more than 1 transition, find the different transitions
# then prompt user to select one. After selecting transition, if there
# are more than one state, prompt user to select a state.
#
# 1. In order to do reselection, we should set "refresh" flag "refresh"
# 2. "refresh" flag brings side effect: 1)it can not distinguish a node
# which has not been expanded and a node which has been expanded. So,
# we have to check it ourself. 2)We have to specify whether a node is
# leaf or branch explicitly. 
#---------------------------------------------------------------------
proc query_sim_node {node_id} {
    global pnamelist procname 
    global simTree simProcList
    global state_num resel_states
    global _node   #_node is used to store the current expanded tree
                   #_node($uid) is used to store the children of $uid

    set rlist {}
    set tlist {}
    set uid [lindex $node_id 0]
    set startstate 0
    set iconname ""

    if {$resel_states == 0} { 
	# If the node has been expanded, return the previous one
	# We set "refresh" flag, so we need check it
	foreach index [array name _node] {
	    if {$index == $uid} {
		set rlist $_node($uid)
		return $rlist
	    }
	} 
        set _node($uid) {}   
    } else {               #if it is reselection, keep the previous info. 
	set rlist $_node($uid)
	set resel_states 0
    } 

    if {$node_id == ""} {
	set startstate 1
	xsb_query get_local_startstate($pnamelist,Id) child 
    } else {
	xsb_query get_local_child($uid,Id,Tran,Cond) child
    }

    if {$child(ntuples) == 0} {   
	return $rlist
    } elseif {$child(ntuples) == 1} {   
	set sel_num 1
	if {$startstate !=1} {
	    # If there are only one transition, there are only two cases: 
	    # 1.user input seq number 2.no input.
	    if {[string first in( $child(1,2)] == 0} {
		set seq [input_seq_num $uid $child(1,2) $child(1,1)]
		xsb_query get_conditional_child($child(1,1),$child(1,2),$seq,$child(1,3),Anschild,Result) anschild
		if {$anschild(1,2) == "false"} { 
		    return $rlist 
		}
		set child(1,1) $anschild(1,1)
		xsb_query translate_input_channel($child(1,2),$seq,Channame) chan_name
		set transname $chan_name(1,1)
	    } else {
		xsb_query translate_trans($uid,$child(1,2),$child(1,1),Channame) chan_name
		set inputlist [lindex $chan_name(1,1) 0]
		set transname [trans_form $inputlist]
#		xsb_query translate_channel($child(1,2),Channame) chan_name
	    }
	} 
    } else {
	# Get different transitions, if more than 1, prompt user to select
	set clist {}
	for {set i 1} {$i <= $child(ntuples)} {incr i} {
	    if {[lsearch -exact $tlist $child($i,2)] == -1} {
		lappend tlist $child($i,2)
		lappend clist $child($i,1)
	    }
	}
	if {[llength $tlist] > 1} {	
	    set tstate [select_transition $uid $tlist $clist]
	} else {
	    set tstate 0 
	}

	# Get the states conresponding to the selected transition
	set sel_states {}
	for {set i 1} {$i <= $child(ntuples)} {incr i} {
	    if {[lindex $tlist $tstate] == $child($i,2)} {
		lappend sel_states $i
	    }
	}

	# If there is input, get the states which satisfy the condition
	set sel_num_list {}
	set transition [lindex $tlist $tstate]

	if {[string first in( [lindex $tlist $tstate]] == 0} {
	    set child_num [lindex $sel_states 0]
	    set seq [input_seq_num $uid $transition $child($child_num,1)]
	    for {set i 0} {$i < [llength $sel_states]} {incr i} {
		set num [lindex $sel_states $i]
		xsb_query get_conditional_child($child($num,1),$child($num,2),$seq,$child($num,3),Anschild,Result) anschild
		if {$anschild(1,2) == "true"} {  
		    set child($num,1) $anschild(1,1)
		    lappend sel_num_list $num
		}
	    }
     	    xsb_query translate_input_channel($transition,$seq,Channame) chan_name
	    set transname $chan_name(1,1)
	} else {
	    set sel_num_list $sel_states
	    xsb_query translate_channel($transition,Channame) chan_name
	    set transname $chan_name(1,1)
	}		 
#	set transname $chan_name(1,1)

	# Prompt user to select state
	if {[llength $sel_num_list] == 0} {
	    return $rlist
	} elseif {[llength $sel_num_list] == 1} {
	    set sel_num [lindex $sel_num_list 0]
	} else {
	    set sel_num [select_state $sel_num_list child]
	}
    }

    # Check whether the state exists
    foreach arrid [array names _node] {
	foreach sel_item $_node($arrid) {
	    if {[lindex $sel_item 0] == $child($sel_num,1)} {	
		# Reselection, a reselected node exists?
		if {$uid == $arrid} {
		    return $rlist
		}
		select_sim_node $child($sel_num,1)
		
		set temp ancestor:
		if {[string first : [lindex $sel_item 1]] != -1} {
		    regexp {([^<]*)<} [lindex $sel_item 1] match state	
		    append temp $state "  <-transition:  " $transname 
		} else {
		    append temp [lindex $sel_item 1] "  <-transition:  " $transname
		}

		# Reselection, a reselected ancestor node exists?
		if {$_node($uid) != {}} {
		    foreach item $_node($uid) {
			if {$temp == [lindex $item 1]} {
			    return $rlist 
			}
		    }
		}
		lappend _node($uid) [list $temp $temp "leaf" $iconname]
		lappend rlist [list $temp $temp "leaf" $iconname]
	        return $rlist
	    }
	} 
    }

    #If the state doesn't exist
    set temp state
    append temp $state_num 
    if {$startstate == 0} {
	append temp "  <-transition:  " $transname
    }

    set state_num [expr $state_num + 1]
    xsb_query get_local_child($child($sel_num,1),Id,Tran,Cond) result
    # If no children, set "leaf", or else, set "branch"
    if {$result(ntuples)==0} {
	set branch "leaf"
    } else {
	set branch "branch"
    	if {($result(ntuples)>1) || ([string first in( $result(1,2)] == 0)} {
	set iconname "iconreselection"
    	}
    }

    lappend rlist [list $child($sel_num,1) $temp $branch $iconname]
    lappend _node($uid) [list $child($sel_num,1) $temp $branch $iconname]
    select_sim_node $child($sel_num,1)
    $simTree component list yview moveto 1
    return $rlist
}


# Prompt user to input seq number
proc input_seq_num {uid trans child} {
    global pstate term
  
    catch {destroy .prompt}
    wm title [toplevel .prompt] "Input Num"
    wm geometry .prompt +200+200

    # We need distinguish variables and constants
    xsb_query get_input_var($uid,$trans,$child,Varname) input_var
    xsb_query get_all_input($uid,$trans,$child,Valuename) input_list

    set term [translate input_var $input_list(1,1)]
    return $term
}

proc translate {input inputl} {
    upvar $input input_var
    global entrylist termnumlist
    global tokenlist term

    set inp [frame .prompt.input_value]
    pack $inp -padx 4 -pady 4 -side top
    label $inp.text -text "Please input variables:" -font "helvetica -16" \
	-foreground blue
    pack $inp.text -side top -padx 4 -pady 4

    set terms $inp.terms
    set coma $inp.coma
    set sentry $inp.entry
    set entrylist {}
    set termnumlist {}

    set input_list [lindex $inputl 0]
    set len [llength $input_list]
    puts $len
    if {$len == 1} {
	set value [lindex $input_list 0]
	set var [check_var $value input_var]
	if {$var == 0} {
	    puts "something wrong in the file"
	} else {
	    set tokenlist {}
	    lappend tokenlist $value
	    puts $tokenlist
	    iwidgets::entryfield $sentry -labeltext $value= \
		-foreground black -insertbackground red -width 8
	    pack $sentry -padx 4 -pady 4 -side left
	    lappend entrylist $sentry
	}
    } else {
	set output [trans_form $input_list]
	set tokenlist [get_tokens $output]
	for {set i 0} {$i < [llength $tokenlist]} {incr i} {	
	    set para [lindex $tokenlist $i]
	    set var [check_var $para input_var]
	    if {$var == 0} {
		label $terms$i -text $para -font "Helvetica -14"
		pack $terms$i -side left
	    } else {
		iwidgets::entryfield $sentry$i -labeltext $para= \
		    -foreground black -insertbackground red -width 8
		pack $sentry$i -padx 4 -pady 4 -side left
		lappend entrylist $sentry$i
		lappend termnumlist $i
	    }
	}
    }

    set btn [frame .prompt.buttons]
    pack $btn -padx 4 -pady 4 -side bottom
    button $btn.ok -text "OK" -command {set term [input_term_ok]}
    pack $btn.ok -side left -expand 1
    tkwait variable pstate
    return $term
}

proc get_tokens {instring} {
    set output {}
    set token ""
   
    set i 0
    while {$i < [string length $instring]} {
	if {[string index $instring $i]=="("} {
	    lappend output $token
	    lappend output (
	    set token ""
	} elseif {[string index $instring $i] == ")"} {
	    lappend output $token
	    lappend output )
	    set token ""
	} elseif {[string index $instring $i] == ","} {
	    lappend output $token
	    lappend output ,
	    set token ""
	} elseif {[string index $instring $i] == "'"} {
	    append token "','"
	    set i [expr $i + 2]
	} else {
	    append token [string index $instring $i]
	}
	set i [expr $i + 1]
    }
    return $output
}

proc trans_form {input} {
	set len [llength $input]
	if {$len == 1} {
	    return $input
	} else {
	    puts [lindex $input 0]
	    if {[lindex $input 0] != ","} {	
		set output [lindex $input 0]
		append output (
	    } else {
		set output '
		append output , '
		append output (
	    }
	    for {set i 1} {$i < [expr $len - 1]} {incr i} {
		set para [lindex $input $i]
		if {[llength $para] > 1} {
		    set out [trans_form $para]
		    append output $out
		    append output ,
		} else {
		    append output $para 
		    append output ,
		}
	    }
	    set last [lindex $input [expr $len - 1]]
	    if {[llength $last] > 1} {
		set out [trans_form $last]
                append output $out 	
	    } else {
		append output $last
	    }
	    append output )	
	}
	puts $output
	return $output
}


# check whether a value is a variable
proc check_var {input_value input_var} {
    upvar $input_var varlist
    
    for {set i 1} {$i <= $varlist(ntuples)} {incr i} {
	if {$input_value == $varlist($i,1)} {
	    return 1
	}
    }
    return 0
}


# after input the term, construct a new term
proc input_term_ok {} {
    global entrylist termnumlist
    global pstate tokenlist
	
    set dflag 1
    for {set i 0} {$i < [llength $entrylist] } {incr i} {
        set para [[lindex $entrylist $i] get]
        if {$para == ""} {
             focus [[lindex $entrylist $i] component entry]
                 set dflag 0
                 break
        }
    }

    if {$dflag == 1} {
	set len [llength $tokenlist]
	if {$len == 1} {
	    set term [[lindex $entrylist 0] get]
	} else {
	    for {set i 0} {$i < $len} {incr i} {
		set pos [lsearch -exact $termnumlist $i]
		if {$pos != -1} {
		    append term [[lindex $entrylist $pos] get] 
		} else {
		    append term [lindex $tokenlist $i]
		}
	    }
	}
	destroy .prompt
	set pstate 1
	return $term
    }
}


proc set_warning_mesg {mesg} {
    catch {destroy .mesg}
    set w [toplevel .mesg]
    wm title $w "Warning"
    wm geometry $w +400+200

    iwidgets::messagebox $w.warning -labeltext "Warning message" \
	-hscrollmode dynamic \
	-vscrollmode dynamic \
	-labelpos n -visibleitems 40x8
    pack $w.warning -padx 4 -pady 4 -fill both -expand yes

    $w.warning type add ERROR -background white -foreground red \
	-font "Helvetica -14"
    $w.warning issue $mesg ERROR

    button $w.ok -text "OK" -command "destroy .mesg"\
	-bd 2 -relief raised
    pack $w.ok -side left -expand 1
}


# Prompt user to select a transition
proc select_transition {uid tran_list child_list} {
    global tstate

    catch {destroy .choice}
    wm title [toplevel .choice] "Selection"
    wm geometry .choice +200+200

    text .choice.t -relief raised -bd 2 -height 1 -width 30
    pack append .choice .choice.t {top expand fillx}
    .choice.t insert end "Select a transition:"

    for {set i 0} {$i < [llength $tran_list]} {incr i} {
	set tran [lindex $tran_list $i]
	set child [lindex $child_list $i]
	if {([string first in( $tran] == 0)||([string first out( $tran] == 0)} {
	    xsb_query translate_trans($uid,$tran,$child,Channame) chan_name
	    set inputlist [lindex $chan_name(1,1) 0]
	    set tran [trans_form $inputlist]
	}
	button .choice.bt$i -text "   Transition: $tran   " \
	    -anchor w -foreground blue -command "set tstate $i"
	pack append .choice .choice.bt$i {top expand fillx}
    }
    tkwait variable tstate
    destroy .choice
    return $tstate
}


# Prompt user to select a state
proc select_state {sel_num_list carray} {
    global simProcList sstate
    upvar $carray child

    catch {destroy .choice}
    wm title [toplevel .choice] "Selection"
    wm geometry .choice +200+200

    text .choice.t -relief raised -bd 2 -height 1 -width 20
    pack append .choice .choice.t {top expand fillx}
    .choice.t insert end "Select a state:" 
    
    set i 1
    foreach sel_num $sel_num_list {
	set cn [lindex $sel_num 0]
	xsb_query get_local_process($child($cn,1),Proc,Line,Char,Var) result

	button .choice.bt$i -text "Choice $i: $child($cn,1)" \
	    -anchor w -foreground blue -command "set sstate $cn"
	pack append .choice .choice.bt$i {top expand fillx}

	set line $result(1,2)
	set char $result(1,3)

	puts stdout $line

	bind .choice.bt$i <Enter> "report $line $char"
	bind .choice.bt$i <Leave> "$simProcList delete 0 end" 
	set i [expr $i +1]
    }
    tkwait variable sstate
    set sel_num $sstate	
    destroy .choice
    return $sel_num
}


# Highlight the corresponding lines
proc report {line char} {
	global simProcList

	$simProcList insert end [list "" $line $char ""]
	highlight_sim_all
}


# reselect node
proc reselect_node {uid} {
    global resel_states
    global simTree _node

    select_sim_node $uid
    set uid1 [lindex $uid 0]
    if {$uid1 == "leaf" || $uid1 == "branch"} {
	set uid1 [lindex $uid 1]
    }

    if [info exists _node($uid1)] {
	set resel_states 1
	$simTree refresh $uid1
	foreach item $_node($uid1) {
	    $simTree collapse [lindex $item 0]
	}
    }
 }


proc expand_select {uid} {
	expand_recursive $uid
}


proc expand_recursive {uid} {
	global simTree

	if {[string first ancestor $uid] == -1} {
		set children [query_sim_node $uid]
		regexp {([^:]*):} $children match child
		puts stdout $child
		$simTree expand $uid
		expand_recursive $child
	}
}


proc highlight_sim_one {p color} {
    	global src_area
    	set line [lindex $p 1]
    	set char [lindex $p 2]
    	$src_area tag add "LINE$color" $line.0 [expr $line+1].0
    	$src_area tag add "CHAR$color" $line.[expr $char-1] $line.$char
}


proc highlight_sim_all {} {
    	global simProcList src_area

    	# clear all existing highlights
    	$src_area tag delete LINEdim CHARdim LINEbright CHARbright
    	set_highlight_color

    	# add new highlights
    	set proclist [$simProcList get 0 end]
    	foreach process $proclist {
        	highlight_sim_one $process dim
    	}
}


proc highlight_sim_bright {{what ""}} {
    	global simProcList src_area

    	if {$what != ""} {
       	    # clear current bright highlight
            $src_area tag delete LINEbright CHARbright
            set_highlight_color
            # bright new highlight
            set process [$simProcList get $what]
            highlight_sim_one $process bright
    	}
}


proc select_sim_node {node_id} {
    global simTree simProcList

    set uid [lindex $node_id 0]
    if {$uid == "leaf" || $uid == "branch"} {
	    set uid [lindex $node_id 1]
    }

    if {[string first ancestor $uid] == -1} {
    	# set current selection & show it
    	$simTree selection clear
    	$simTree selection add $uid

    	# clear the process list
    	catch { $simProcList delete 0 end }
    
	# get the symbol table
    	xsb_query get_local_process($uid,Proc,Line,Char,Vars) result

    	# fill the process list
    	for {set x 1} {$x <= $result(ntuples)} {incr x } {
            set process     $result($x,1)
            set line        $result($x,2)
            set char        $result($x,3)
            set variables   [string range $result($x,4) 1 \
                  [expr [string length $result($x,4)] - 2]]

            $simProcList insert end [list $process $line $char $variables]
    	}

    	# highlight the lines
    	highlight_sim_all
    }
}

