#!/bin/sh
#
# File: XMC/install.sh
#
# Script to install the XMC system.
#
# $Id: install.sh,v 1.3 2000/11/05 22:15:11 ydong Exp $
#

# You may want to change the default setting of the following three
# variables before you run this script (so you do not have to type in
# them interactively).

CC=gcc

# ===============================================
# You SHOULD NOT change anything below this line.
# ===============================================

XMC_VERSION=1.0
XMC_URL=http://www.cs.sunysb.edu/~lmc/
XMC_PATH=`pwd`
INSTALL_LOG="`pwd`/Installation_summary"

# Some utility functions
# --------------------------------------------
get_str()
{
    $ECHO "$* \c"
    read ANS
}

# Set ECHO
# --------------------------------------------
# Use Sysv echo if possible
if [ -x /usr/5bin/echo ]
then
    ECHO=/usr/5bin/echo				# SunOS SysV echo
elif [ -z "`(echo -e a) 2>/dev/null | fgrep e`" ]
then
    ECHO="echo -e"				# GNU bash, etc.
else
    ECHO=echo					# generic SysV
fi
if [ "$OS_SYS" = QNX ]
then
    ECHO=echo
fi

cat <<EOF
========================================================================
 Before running the installation, you must have installed
   XSB     -- a tabled logic programming system, and
   itcl/tk -- a GUI toolkit
 Please see the README file for details on obtaining and installing the 
 packages.
========================================================================
EOF

while :
do
    get_str "Continue? (y/n)\n  [y]"
    if [ -z "$ANS" -o "$ANS" = "y" -o "$ANS" = "n" -o \
	 "$ANS" = "Y" -o "$ANS" = "N" ]; then
	break
    fi
done

$ECHO

if [ "$ANS" = "n" -o "$ANS" = "N" ]; then
    exit 0
fi

$ECHO "\nInstalling XMC version $XMC_VERSION"
$ECHO "=========================="
$ECHO "\nPlease visit the XMC web site: $XMC_URL"

# Get the full path name of the XSB executable
# --------------------------------------------
out=`which xsb 2>&1`
case "$out" in
    *xsb)	XSB_EXEC="$out";;
    which*)	XSB_EXEC="/usr/local/bin/xsb";;
esac
while :
do
    get_str "\nEnter the full path name of the XSB executable\n  [${XSB_EXEC}]:"
    if [ -z "$ANS" ]
    then
	ANS=$XSB_EXEC
    fi

    if [ -x "$ANS" ]
    then
	XSB_EXEC=$ANS
	break
    else
	$ECHO "  Error: $ANS does not exist, please try again."
    fi
done

# Get the C compiler
# --------------------------------------------
while :
do
    get_str "\nEnter the C compiler you want to use (gcc or cc)\n\
(Normally it should be the compiler used to compile XSB)\n\
  [${CC}]:"
    if [ -z "$ANS" ]
    then
	ANS=$CC
    fi

    if [ "$ANS" = "gcc" -o "$ANS" = "cc" ]
    then
	CC=$ANS
	break
    else
	$ECHO "  Error: You must choose gcc or cc."
    fi
done

# Get the full path name of ITKWISH
# --------------------------------------------
out=`which itkwish 2>&1`
case "$out" in
    *itkwish)	ITKWISH_EXEC="$out";;
    which*)	out=`which itkwish3.0 2>&1`;
		case "$out" in
		    *itkwish3.0) ITKWISH_EXEC="$out";;
		    which*)	 ITKWISH_EXEC="/usr/local/bin/xsb";;
		esac;;
esac
while :
do

    get_str "\nDo you want to install GUI? (y/n)  [y]"
    if [ "$ANS" = "n" -o "$ANS" = "N" ]; then
	break
    fi

    if [ -z "$ANS" -o "$ANS" = "y" -o "$ANS" = "Y" ]; then
	get_str "\nEnter the full path name of itkwish\n  [${ITKWISH_EXEC}]:"
	if [ -z "$ANS" ]
	then
	    ANS=$ITKWISH_EXEC
	fi
	
	if [ -x "$ANS" ]
	then
	    ITKWISH_EXEC=$ANS
	    break
	else
	    $ECHO "  Error: $ANS does not exist, please try again."
	fi
    fi
done

rm -f $INSTALL_LOG

$ECHO "\nXMC Installation Summary (`date`)" > $INSTALL_LOG
$ECHO "-------------------------------------------------------" >> $INSTALL_LOG
$ECHO "`uname -a`\n" >> $INSTALL_LOG
$ECHO "XSB_EXEC=$XSB_EXEC" >> $INSTALL_LOG
$ECHO "ITKWISH_EXEC=$ITKWISH_EXEC" >> $INSTALL_LOG
$ECHO "XMC_PATH=$XMC_PATH" >> $INSTALL_LOG
$ECHO "CC=$CC" >> $INSTALL_LOG

# Create startup script
# --------------------------------------------
cd System
$ECHO "\nCreating Prolog components..." | tee -a $INSTALL_LOG
rm -f xmc.P
$ECHO ":- assert(xmc_directory('$XMC_PATH/System'))." > xmc.P
cat xmc.proto >> xmc.P

$ECHO "Creating startup script..." | tee -a $INSTALL_LOG

cd ..
if [ ! -d bin ] ; then
    mkdir bin
fi
cd bin
rm -f xmc

$ECHO "#!/bin/sh" > xmc
$ECHO "## This file is automatically generated by install.sh" >> xmc
$ECHO >> xmc
$ECHO "$XSB_EXEC -e \"consult('$XMC_PATH/System/xmc').\" \$*" >> xmc

chmod +x xmc

# Create xmc-gui
# --------------------------------------------
$ECHO "Creating xmc-gui..." | tee -a $INSTALL_LOG
$ECHO "#!/bin/sh" > xmc-gui
$ECHO "## This file is automatically generated by install.sh" >> xmc-gui
$ECHO >> xmc-gui
$ECHO "ITKWISH_EXEC=$ITKWISH_EXEC" >> xmc-gui
$ECHO "XSB_EXEC=$XSB_EXEC" >> xmc-gui
$ECHO "XMC_PATH=$XMC_PATH" >> xmc-gui
$ECHO "export XSB_EXEC" >> xmc-gui
$ECHO "export XMC_PATH" >> xmc-gui
$ECHO >> xmc-gui
$ECHO "\$ITKWISH_EXEC \$XMC_PATH/GUI/xmc.tcl > /dev/null" >> xmc-gui

chmod +x xmc-gui

# Make Prolog components
# --------------------------------------------
cd ../System
$ECHO "Creating all other necessary components..." | tee -a $INSTALL_LOG

make XSB=$XSB_EXEC CC=$CC 2>&1 | tee -a $INSTALL_LOG

$ECHO "Installation finished." >> $INSTALL_LOG
$ECHO "-------------------------------------------------------" >> $INSTALL_LOG

# Registration
# --------------------------------------------
$ECHO ""
$ECHO "The installation process is now complete.  The installation summary is"
$ECHO "stored in $INSTALL_LOG."
$ECHO ""
$ECHO "We shall greatly appreciate if you would agree to send us this log."
$ECHO "Installation logs help the XMC group to keep tracek of the usage of the"
$ECHO "system on different architectures and to isolate problems more easily."
$ECHO ""
$ECHO "The log will be sent automatically to lmc@cs.sunysb.edu."
$ECHO "Would you like to send us the installation log? (y/n)"
$ECHO "  [y] \c"

read ANS

if test "$ANS" != "n" -a "$ANS" != "no" -a "$ANS" != "N" ; then
    cat $INSTALL_LOG | mail lmc@cs.sunysb.edu
    $ECHO "\nThank you!"
fi

$ECHO
$ECHO "Installation done.  Now you can run XMC using the shell script:"
$ECHO "\t${XMC_PATH}/bin/xmc, or"
$ECHO "\t${XMC_PATH}/bin/xmc-gui\n"
